package edu.harvard.mgh.purcell.gplink.mainGUI;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.io.File;
import java.io.IOException;

import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;
import javax.swing.JMenuItem;

import ch.ethz.ssh2.SCPClient;

import edu.harvard.mgh.purcell.gplink.data.Project;
import edu.harvard.mgh.purcell.gplink.base_form.GeneralUtil;
/**
 * A simple popup for editing or deleting operations.
 * @author Kathe Todd-Brown
 */
@SuppressWarnings("serial")
public class TreePopup extends JPopupMenu {

	
	Project data;
	String opName;
	MainFrame frame;
	String filename;
	String remoteFilename;
	
	/**
	 * call an operation editing dialog
	 */
	private ActionListener exitOp = new ActionListener(){
		public void actionPerformed(ActionEvent event){
			new EditOpDialog(opName, frame);
		}
	};
	
	/**
	 * create a new operation based on opName
	 */
	private ActionListener createOp = new ActionListener(){
		public void actionPerformed(ActionEvent event){
			new PlinkExicute(frame, data.getOpCommand(opName));
		}
	};
	
	/**
	 * delete an operation
	 */
	private ActionListener deleteOp = new ActionListener(){
		public void actionPerformed(ActionEvent event){
			new DeleteOpDialog(frame, new String [] { opName});
		}
	};
	
	/**
	 * open in haploview
	 */
	private ActionListener openHaplo = new ActionListener(){

		public void actionPerformed(ActionEvent e) {
			
			/*
			 * *************************************************************
			 * Deal with ped/info combo since it has a seperate command line
			 * *************************************************************
			 */
			if (filename.endsWith(".ped") || filename.endsWith(".info"))
			{
				String filename2 = "";
				if ( filename.endsWith(".ped") ){
					filename2 =  filename.subSequence(0, 
							filename.length() - 4) + ".info";
					if ( ! new File(filename2).exists() ){
						new ErrorDialog(frame,
								"Can not open in Haploview without " 
								+ filename2);
							return;
					}										
				} else {
					filename2 =  filename.subSequence(0, filename.length() - 5) + ".ped";
					if ( ! new File(filename2).exists() ){
						new ErrorDialog(frame,
								"Can not open in Haploview without " + filename2);
							return;
					}
					
					// Swap so that filename = ped, filename2 = info
					String tmp = filename;
					filename = filename2;
					filename2 = tmp;
				}

				
				String [] hapOptions = GeneralUtil.stripAndSplit(
						data.getHaploviewAppend());
				String [] command = new String [ 7 +
				         hapOptions.length];
				command[0] = "java";
				command[1] = "-jar";
				command[2] = data.getHaploviewPath();
				command[3] = "-p";
				command[4] = filename;
				command[5] = "-i";
				command[6] = filename2;
				
				//append the projects haploview
				if(!hapOptions[0].equals(null)){
					for(int i = 0; i < hapOptions.length; i ++){
						command[i+7] = hapOptions[i];
					}
				}
				
				//send it off to the os
				try{	
					@SuppressWarnings("unused")
					Process haplo = Runtime.getRuntime().exec(command);
					
				} catch (IOException e1) {
		        	new ErrorDialog(frame,
		    	        	"Error trying to open the following files " +
    	        			"in haploview " +
    	        			"[ "+filename+" ], [ "+filename2+" ].\n" +
    	        			"openHaploview:TreePopup.java");
		        }
				//we are done!
		        return;
			}
				
			
					
			/*
			 * ****************************************** 
			 * Deal with non-SNP files in Haploview
			 * ******************************************
			 */
			

			if(filename.endsWith(".mds")
					|| filename.endsWith(".cluster2")
					|| filename.endsWith(".imendel")
					|| filename.endsWith(".fmendel")
					|| filename.endsWith(".mendel")
					|| filename.endsWith(".imiss")
					|| filename.endsWith(".het"))		
			{
			
			if(new File(filename).exists() == false )
			{
				if(data.isRemote(false))
					new ErrorDialog(frame, 
						"[ " + filename + " ] does not exist, " +
								"can not open Haploview.\n" +
						"Possibly not downloaded from remote server.\n" +
						"openHaplo:TreePopup.java");
					
				else
					new ErrorDialog(frame, 
						"[ " + filename + " ] does not exist, " +
								"can not open Haploview.\n" +
						"openHaplo:TreePopup.java");
				return;
			}
						
			String [] hapOptions = GeneralUtil.stripAndSplit(
					data.getHaploviewAppend());
			String [] command = new String [ 6 +
			         hapOptions.length];
			command[0] = "java";
			command[1] = "-jar";
			command[2] = data.getHaploviewPath();
			command[3] = "-plink";
			command[4] = filename;
			command[5] = "-nonSNP";
			//add any extra's to the haploview command
			if(!hapOptions[0].equals(null)){
				for(int i = 0; i < hapOptions.length; i ++){
					command[i+6] = hapOptions[i];
				}
			}
			try{
				@SuppressWarnings("unused") 
				Process haplo = Runtime.getRuntime().exec(command);
			} catch (IOException e1) {
	        	new ErrorDialog(frame,
	        			"Error trying to open the following files in Haploview " +
	        			"[ " + filename + "]. \n" +
	        			"openHaplo:TreePopup.java");
	        }
			
			return;
			}
			
			
			
			
			/*
			 * ****************************************** 
			 * Deal with remaining files (SNP results)
			 * ******************************************
			 */
			
			String infile = "";
			String [] allInfiles = data.getOpFiles(opName, Project.INFILE_KEY)[0];
			for(int i = 0; i < allInfiles.length; i++){
				if(allInfiles[i].endsWith(".map")){
					if(data.isRemote(false)){
						infile = new File(data.getRoot(), GeneralUtil.fileName(allInfiles[i])).getAbsolutePath();
					}else
						infile = allInfiles[i];
				}
				if(allInfiles[i].endsWith(".bim")){
					if(data.isRemote(false)){
						infile = new File(data.getRoot(), GeneralUtil.fileName(allInfiles[i])).getAbsolutePath();
					}else
						infile = allInfiles[i];
				}
			}
			
			if(new File(filename).exists() == false || new File(infile).exists() == false)
			{
				if(data.isRemote(false))
					new ErrorDialog(frame, 
						"[ " + filename + " ] and/or [ "+ infile + " ] does not exist, " +
								"can not open haploview.\n" +
						"Possibly not downloaded from remote server.\n" +
						"openHaplo:TreePopup.java");
					
				else
					new ErrorDialog(frame, 
						"[ " + filename + " ] and/or [ "+ infile + " ] does not exist, " +
								"can not open haploview.\n" +
						"openHaplo:TreePopup.java");
				return;
			}
			
			String [] hapOptions = GeneralUtil.stripAndSplit(
					data.getHaploviewAppend());
			String [] command = new String [ 7 +
			         hapOptions.length];
			command[0] = "java";
			command[1] = "-jar";
			command[2] = data.getHaploviewPath();
			command[3] = "-plink";
			command[4] = filename;
			command[5] = "-map"; 
			command[6] = infile;
			//add any extra's to the haploview command
			if(!hapOptions[0].equals(null)){
				for(int i = 0; i < hapOptions.length; i ++){
					command[i+7] = hapOptions[i];
				}
			}
			try{
				@SuppressWarnings("unused") 
				Process haplo = Runtime.getRuntime().exec(command);
			} catch (IOException e1) {
	        	new ErrorDialog(frame,
	        			"Error trying to open the following files in haploview " +
	        			"[ " + filename + "], [ " + infile + "]. \n" +
	        			"openHaplo:TreePopup.java");
	        }
			

			

			
		
		}
		
	};
	
	/**
	 * Launch the user specified file viewer.
	 */
	private ActionListener viewAlternate = new ActionListener(){
		@SuppressWarnings("static-access")
		public void actionPerformed(ActionEvent event){
			try{
				//open the file in the editor
				String s = data.getEditor() + " " 
					+ GeneralUtil.processFilename(filename);
				@SuppressWarnings("unused") 
				Process plink = Runtime.getRuntime().exec(GeneralUtil.stripAndSplit(s),
						null, data.getRoot());
			}
	        catch (IOException e) {
	        	JOptionPane.showMessageDialog(frame,
	    	        	//new ErrorMessageDialog(frame, 
	        			"Error trying to open selected file [ "
	        			+ filename + " ] in " +
	        			"alternate editor [ " + data.getEditor() + " ].\n",
	        			"gPLINK error",
	        			JOptionPane.ERROR_MESSAGE);
	        }
		}
	};
	
	/**
	 * Launch the default file viewer
	 */
	private ActionListener viewDefault = new ActionListener(){
		public void actionPerformed(ActionEvent event){
			try{
				//open the file in the user spcifed filename
				String s = data.getDefaultEditor() + " "
					+ GeneralUtil.processFilename(filename);
				@SuppressWarnings("unused") 
				Process plink = Runtime.getRuntime().exec(GeneralUtil.stripAndSplit(s),
						null, data.getRoot());
			}
	        catch (IOException e) {
	        	JOptionPane.showMessageDialog(frame,
	    	        	//new ErrorMessageDialog(frame, 
	        			"Error trying to open selected file [ "
	        			+ filename + " ] in " +
	        			"default editor [ " + data.getDefaultEditor() + " ].\n",
	        			"gPLINK error",
	        			JOptionPane.ERROR_MESSAGE);
	        }
		}
	};
	
	/**
	 * Download the file via scp.
	 */
	private ActionListener downloadFile = new ActionListener(){
		public void actionPerformed(ActionEvent arg0) {
			SCPClient scopy = new SCPClient(frame.data.getConn());
			try {
				scopy.get(GeneralUtil.makeSpacesHard(remoteFilename),
						data.getRoot().getAbsolutePath());
			} catch (IOException e) {
				new ErrorDialog(frame, "Error trying to download\n" +
						"[" + GeneralUtil.makeSpacesHard(remoteFilename) +"]\n"
						+"downloadFile:TreePopup.java");
			}
		}
	};
	
	/**
	 * Create a simple popup attached to the main frame and pass in the 
	 * needed data.
	 * @param givenFrame the main frame that the popup is attached to
	 * @param givenData the project that holds the data for all operations
	 * @param givenOpName the name of the operation we are editing
	 * @param selectedName the name of the file that is selected
	 */
	public TreePopup(MainFrame givenFrame, Project givenData, String givenOpName,
			String selectedName){
		super();
		
		opName = givenOpName;
		data = givenData;
		frame = givenFrame;
		filename = selectedName;
		
		//point the filename to the local copy of the file
		if(data.isRemote(false)){
			remoteFilename = filename;
			File remoteCopy = new File(data.getRoot(), GeneralUtil.fileName(filename));
			filename = remoteCopy.getAbsolutePath();
		}
		
		/*
		 * create the menu items
		 */
		JMenuItem edit = new JMenuItem("Edit");
		JMenuItem create = new JMenuItem("Create New");
		JMenuItem delete = new JMenuItem("Delete");
		JMenuItem open = new JMenuItem("Open in default viewer");
		JMenuItem openAlter = new JMenuItem("Open in alternate viewer");
		JMenuItem openInHaplo = new JMenuItem("Open in Haploview");
		JMenuItem download = new JMenuItem("Download file");
			
		open.addActionListener(viewDefault);
		openAlter.addActionListener(viewAlternate);
		openInHaplo.addActionListener(openHaplo);
		edit.addActionListener(exitOp);
		create.addActionListener(createOp);
		delete.addActionListener(deleteOp);
		download.addActionListener(downloadFile);
		
		/*
		 * Populate the menu
		 */
		if(new File(filename).exists()){
			add(open);
			if(!data.getEditor().equals(""))
				add(openAlter);
			if(hapoloviewOK(filename) 
					&& new File(data.getHaploviewPath()).exists())
				add(openInHaplo); 
		} else if(data.isRemote(true) && remoteFilename.contains("/")){
			add(download);
		}
		
		if(data.hasMetaLock() == true){
			add(edit);
			if(data.getBrowse() == false)
				add(create);
			add(delete);
		}
		
		pack();
	}

	/**
	 * Checks to see if a file can be opened in haploview
	 * @param filename string that has a file name
	 * @return true if the file name extention matches a haploview ok list,
	 * false othewise
	 */
	private boolean hapoloviewOK(String filename){
		return (filename.endsWith(".adjusted")
				|| filename.endsWith(".ped")
				|| filename.endsWith(".info")			
				|| filename.endsWith(".assoc")
				|| filename.endsWith(".assoc.mperm")
				|| filename.endsWith(".assoc.perm")
				|| filename.endsWith(".cmh")
				|| filename.endsWith(".cmh2")
				|| filename.endsWith(".frq")
				|| filename.endsWith(".hwe")
				|| filename.endsWith(".lmendel")
				|| filename.endsWith(".lmiss")
				|| filename.endsWith(".missing")
				|| filename.endsWith(".missing.hap")
				|| filename.endsWith(".linear")
				|| filename.endsWith(".logistic")
				|| filename.endsWith(".genotypic")
				|| filename.endsWith(".model")
				|| filename.endsWith(".qassoc")
				|| filename.endsWith(".qassoc.gxe")
				|| filename.endsWith(".tdt")
				|| filename.endsWith(".tdt.mperm")
				|| filename.endsWith(".tdt.perm")
				|| filename.endsWith(".mds")
				|| filename.endsWith(".imiss")
				|| filename.endsWith(".het")	);
	}
}
